#vider la mmoire
rm(list=ls())

#importer les donnes
setwd("D:/DataMining/Databases_for_mining/dataset_for_soft_dev_and_comparison/boosting_randomforest")
image_all <- read.table("image.txt",sep="\t",dec=".",header=TRUE)
print(summary(image_all))

#partition apprentissage et test
image_train <- image_all[image_all$sample=="train",1:20]
image_test <- image_all[image_all$sample=="test",1:20]

print(summary(image_train$REGION_TYPE))
print(summary(image_test$REGION_TYPE))

#fonction d'valuation
error_rate <- function(yobs,ypred){
  #matrice de confusion
  mc <- table(yobs,ypred)
  #taux d'erreur
  err <- 1.0 - sum(diag(mc))/sum(mc)
  return(err)
}

#arbre de dcision
library(rpart)
arbre_1 <- rpart(REGION_TYPE ~ ., data = image_train)
print(arbre_1)

#prdiction sur chantillon test
pred_1 <- predict(arbre_1,newdata=image_test,type="class")

#taux d'erreur
print(error_rate(image_test$REGION_TYPE,pred_1))

#dcision stump
param_stump = rpart.control(cp=0,maxdepth=1,minsplit=2,minbucket=1)
arbre_2 <- rpart(REGION_TYPE ~ ., data = image_train,control=param_stump)
print(arbre_2)

#prdiction et taux d'erreur
pred_2 <- predict(arbre_2,newdata=image_test,type="class")
print(error_rate(image_test$REGION_TYPE,pred_2))

#arbre profond
param_deep = rpart.control(cp=0,maxdepth=30,minsplit=2,minbucket=1)
arbre_3 <- rpart(REGION_TYPE ~ ., data = image_train,control=param_deep)
print(arbre_3)

#prdiction et taux d'erreur
pred_3 <- predict(arbre_3,newdata=image_test,type="class")
print(error_rate(image_test$REGION_TYPE,pred_3))

#librairie adabag
library(adabag)

#bagging
bag_1 <- bagging(REGION_TYPE ~ ., data = image_train,mfinal=20)

#prdiction
predbag_1 <- predict(bag_1,newdata = image_test)

#taux d'erreur
print(error_rate(image_test$REGION_TYPE,predbag_1$class))

#premier arbre
print(bag_1$trees[[1]])

#importance
print(sort(bag_1$importance,decreasing=TRUE))

#sortie graphique
importanceplot(bag_1,cex.names=0.5,horiz=TRUE)

#bagging avec un seul arbre
param = rpart.control(maxsurrogate=20)
bag_seul <- bagging(REGION_TYPE ~ ., data = image_train,mfinal=1)

#l'arbre
print(bag_seul$trees[[1]])

#importance
print(bag_seul$importance)

#bagging de decision stump
bag_stump <- bagging(REGION_TYPE ~ ., data = image_train,mfinal=20, control=param_stump)

#prdiction
predbag_stump <- predict(bag_stump,newdata = image_test)

#taux d'erreur
print(error_rate(image_test$REGION_TYPE,predbag_stump$class))

#bagging arbre plus profond
bag_deep <- bagging(REGION_TYPE ~ ., data = image_train,mfinal=20, control=param_deep)

#prdiction
predbag_deep <- predict(bag_deep,newdata = image_test)

#taux d'erreur
print(error_rate(image_test$REGION_TYPE,predbag_deep$class))

#nombre m d'arbres  tester
m_a_tester <- c(1,5,10,20,50,100,200)

#apprentissage-test
train_test_bag <- function(m){
  bag <- bagging(REGION_TYPE ~ .,data=image_train,mfinal=m,control=param_deep)
  predbag <- predict(bag,newdata = image_test)
  return(error_rate(image_test$REGION_TYPE,predbag$class))
}

#valuation 20 fois de chaque valeur de m
result <- replicate(20,sapply(m_a_tester,train_test_bag))

#graphique
plot(m_a_tester,apply(result,1,mean),xlab="m",ylab="Err. rate",type="b")

#random forest
library(randomForest)
rf_1 <- randomForest(REGION_TYPE ~ ., data = image_train, ntree = 20)

#prdiction
predrf_1 <- predict(rf_1,newdata=image_test,type="class")

#erreur
print(error_rate(image_test$REGION_TYPE,predrf_1))

#matrice de confusion out-of-bag
print(rf_1$confusion)

#erreur out-of-bag
print(1-sum(diag(rf_1$confusion))/sum(rf_1$confusion))

#apparition des variables
print(data.frame(cbind(colnames(image_train)[2:20],varUsed(rf_1))))

#importance des variables
varImpPlot(rf_1)

#accs au 1er arbre
print(getTree(rf_1,1))

#nombre m d'arbres  tester
m_a_tester <- c(1,5,10,20,50,100,200)

#apprentissage-test
train_test_rf <- function(m){
  rf <- randomForest(REGION_TYPE ~ .,data=image_train,ntree=m)
  predrf <- predict(rf,newdata = image_test)
  return(error_rate(image_test$REGION_TYPE,predrf))
}

#valuation 20 fois de chaque valeur de m
result <- replicate(20,sapply(m_a_tester,train_test_rf))

#graphique
plot(m_a_tester,apply(result,1,mean),xlab="m",ylab="Err. rate",type="b")

#librairie adabag
library(adabag)

#boosting
bo_1 <- boosting(REGION_TYPE ~ ., data = image_train,mfinal = 20, boos= FALSE)

#prdiction
predbo_1 <- predict(bo_1,newdata = image_test)

#taux d'erreur
print(error_rate(image_test$REGION_TYPE,predbo_1$class))

#boosting de decision stump
bo_stump <- boosting(REGION_TYPE ~ ., data = image_train,mfinal=20, boos=FALSE, coeflearn= 'Zhu', control=param_stump)

#prdiction
predbo_stump <- predict(bo_stump,newdata = image_test)

#taux d'erreur
print(error_rate(image_test$REGION_TYPE,predbo_stump$class))

#boosting arbre plus profond
bo_deep <- boosting(REGION_TYPE ~ ., data = image_train,mfinal=20, boos=FALSE, coeflearn= 'Zhu', control=param_deep)

#prdiction
predbo_deep <- predict(bo_deep,newdata = image_test)

#taux d'erreur
print(error_rate(image_test$REGION_TYPE,predbo_deep$class))

#nombre m d'arbres  tester
m_a_tester <- c(1,5,10,20,50,100,200)

#apprentissage-test
train_test_boosting <- function(m){
  bo <- boosting(REGION_TYPE ~ .,data=image_train,mfinal=m,coeflearn='Zhu')
  predbo <- predict(bo,newdata = image_test)
  return(error_rate(image_test$REGION_TYPE,predbo$class))
}

#valuation 20 fois de chaque valeur de m
result <- replicate(20,sapply(m_a_tester,train_test_boosting))

#graphique
plot(m_a_tester,apply(result,1,mean),xlab="m",ylab="Err. rate",type="b")
